/**
 * 
 */
package org.ow2.petals.jmx;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import javax.management.Attribute;
import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.InvalidAttributeValueException;
import javax.management.MBeanAttributeInfo;
import javax.management.MBeanException;
import javax.management.MBeanInfo;
import javax.management.MBeanServerConnection;
import javax.management.ObjectName;
import javax.management.ReflectionException;

import org.ow2.petals.jmx.exception.AttributeErrorException;

/**
 * @author aruffie
 * 
 */
public class ConfigurationInstallerComponentClient extends AbstractServiceClient {

    private MBeanInfo mbeanInfo;

    private ObjectName objectName;

    public ConfigurationInstallerComponentClient(final ObjectName objectName,
            final String petalsDomain, final MBeanServerConnection beanServerConnection,
            final MBeanInfo mbeanInfo) {
        super(petalsDomain, beanServerConnection);
        this.mbeanInfo = mbeanInfo;
        this.objectName = objectName;
    }

    public Object getAttribute(final String attribute) throws AttributeErrorException {
        try {
            return this.mBeanServerConnection.getAttribute(this.objectName, attribute);
        } catch (AttributeNotFoundException e) {
            throw new AttributeErrorException(e);
        } catch (InstanceNotFoundException e) {
            throw new AttributeErrorException(e);
        } catch (MBeanException e) {
            throw new AttributeErrorException(e);
        } catch (ReflectionException e) {
            throw new AttributeErrorException(e);
        } catch (IOException e) {
            throw new AttributeErrorException(e);
        }
    }

    public void setAttribute(final String name, final Object value) throws AttributeErrorException {
        try {
            this.mBeanServerConnection.setAttribute(this.objectName, new Attribute(name, value));
        } catch (AttributeNotFoundException e) {
            throw new AttributeErrorException(e);
        } catch (InstanceNotFoundException e) {
            throw new AttributeErrorException(e);
        } catch (MBeanException e) {
            throw new AttributeErrorException(e);
        } catch (ReflectionException e) {
            throw new AttributeErrorException(e);
        } catch (IOException e) {
            throw new AttributeErrorException(e);
        } catch (InvalidAttributeValueException e) {
            throw new AttributeErrorException(e);
        }
    }

    public Map<MBeanAttributeInfo, Object> getAttributes() throws AttributeErrorException {
        MBeanAttributeInfo[] attrs = null;
        final Map<MBeanAttributeInfo, Object> attributes = new HashMap<MBeanAttributeInfo, Object>();
        try {
            attrs = this.mbeanInfo.getAttributes();
            for (final MBeanAttributeInfo info : attrs) {
                attributes.put(info, this.mBeanServerConnection.getAttribute(this.objectName, info
                        .getName()));
            }
        } catch (AttributeNotFoundException e) {
            throw new AttributeErrorException(e);
        } catch (InstanceNotFoundException e) {
            throw new AttributeErrorException(e);
        } catch (MBeanException e) {
            throw new AttributeErrorException(e);
        } catch (ReflectionException e) {
            throw new AttributeErrorException(e);
        } catch (IOException e) {
            throw new AttributeErrorException(e);
        }
        return attributes;
    }

    public void setAttributes(final Map<MBeanAttributeInfo, Object> attributes)
            throws AttributeErrorException {
        try {
            for (final Entry<MBeanAttributeInfo, Object> info : attributes.entrySet()) {
                this.mBeanServerConnection.setAttribute(this.objectName, new Attribute(info
                        .getKey().getName(), info.getValue()));
            }
        } catch (AttributeNotFoundException e) {
            throw new AttributeErrorException(e);
        } catch (InstanceNotFoundException e) {
            throw new AttributeErrorException(e);
        } catch (MBeanException e) {
            throw new AttributeErrorException(e);
        } catch (ReflectionException e) {
            throw new AttributeErrorException(e);
        } catch (IOException e) {
            throw new AttributeErrorException(e);
        } catch (InvalidAttributeValueException e) {
            throw new AttributeErrorException(e);
        }
    }

}
