/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */

package org.ow2.petals.jmx;

import java.io.IOException;
import java.util.Map;
import java.util.Set;

import javax.management.MBeanServerConnection;
import javax.management.ObjectName;

import org.ow2.petals.jmx.api.IPetalsAdminService;
import org.ow2.petals.jmx.exception.PerformActionErrorException;
import org.ow2.petals.jmx.exception.PetalsAdminDoesNotExistException;
import org.ow2.petals.jmx.exception.PetalsAdminServiceErrorException;

import static org.ow2.petals.jmx.api.IPetalsAdminService.MBean.PETALSADMIN_OBJECT_NAME;
import static org.ow2.petals.jmx.api.IPetalsAdminService.OperationNames.RETRIEVE_TOPOLOGY;
import static org.ow2.petals.jmx.api.IPetalsAdminService.OperationNames.SHUTDOWN;
import static org.ow2.petals.jmx.api.IPetalsAdminService.OperationNames.STOP;

/**
 * Client API of the JMX Petals Admin service.
 * 
 * @author rnaudin - EBM Websourcing
 * 
 */
public class PetalsAdminServiceClient extends AbstractServiceClient implements
        IPetalsAdminService {

    /**
     * Creates a new instance of {@link PetalsAdminServiceClient}
     * 
     * @param petalsDomain
     * @param beanServerConnection
     * @param isPetalsServer20Older
     * @throws PetalsAdminDoesNotExistException
     * @throws PetalsAdminServiceErrorException
     */
    @SuppressWarnings("unchecked")
    public PetalsAdminServiceClient(final String petalsDomain,
            final MBeanServerConnection beanServerConnection)
            throws PetalsAdminDoesNotExistException, PetalsAdminServiceErrorException {
        super(petalsDomain, beanServerConnection);

        try {
            final Set<ObjectName> objNames = this.mBeanServerConnection.queryNames(
                    PETALSADMIN_OBJECT_NAME, null);
            if (objNames != null && objNames.size() == 1) {
                this.mbeanName = objNames.iterator().next();
            } else {
                throw new PetalsAdminDoesNotExistException();
            }
        } catch (IOException e) {
            throw new PetalsAdminServiceErrorException(e);
        }
    }

    /**
     * Stop the PEtALS container.
     * 
     * @see PetalsAdminInterface#stopContainer()
     * @throws PerformActionErrorException
     */
    public void stopContainer() throws PetalsAdminServiceErrorException {
        try {
            this.performAction(STOP, null, null);
        } catch (PerformActionErrorException e) {
            throw new PetalsAdminServiceErrorException(e);
        }

    }

    /**
     * Shut down the PEtALS container.
     * 
     * @see PetalsAdminInterface#shutdownContainer()
     * @throws PerformActionErrorException
     */
    public void shutdownContainer() throws PetalsAdminServiceErrorException {
        try {
            this.performAction(SHUTDOWN, null, null);
        } catch (PerformActionErrorException e) {
            throw new PetalsAdminServiceErrorException(e);
        }
    }

    @SuppressWarnings("unchecked")
    public Set<Map<String, String>> retrieveTopology() throws PetalsAdminServiceErrorException {
        final Set<Map<String, String>> result;

        try {
            result = (Set<Map<String, String>>) this.performAction(RETRIEVE_TOPOLOGY, null, null);
        } catch (PerformActionErrorException e) {
            throw new PetalsAdminServiceErrorException(e);
        } catch (ClassCastException e) {
            throw new PetalsAdminServiceErrorException("Unexpected result type");
        }

        return result;
    }
}
